#!/bin/sh
#
# /etc/rc.d/rc.S:  System initialization script.
#
# Mostly written by:  Patrick J. Volkerding, <volkerdi@slackware.com>
#

echo ">>> Running S-script: $0"

PATH=/sbin:/usr/sbin:/bin:/usr/bin

# Mount /proc right away:
/sbin/mount -v proc /proc -n -t proc

# If 'nohotplug' was given at boot, or rc.hotplug has been turned off
# (is not executable), then shut off hotplugging in the kernel now.
# Turning off hotplug is *not* recommended, and will break some things.
if [ -w /proc/sys/kernel/hotplug ]; then
  echo "/dev/null" > /proc/sys/kernel/hotplug
fi

# Mount sysfs next, if the kernel supports it:
if [ -d /sys ]; then
  if cat /proc/filesystems | grep -w sysfs 1> /dev/null 2> /dev/null ; then
    if ! cat /proc/mounts | grep -w sysfs 1> /dev/null 2> /dev/null ; then
      /sbin/mount -v sysfs /sys -n -t sysfs
    fi
  fi
fi

# Enable swapping:
/sbin/swapon -a

# Test to see if the root partition is read-only, like it ought to be.
READWRITE=no
if touch /fsrwtestfile 2>/dev/null; then
  rm -f /fsrwtestfile
  READWRITE=yes
else
  echo "Testing root filesystem status:  read-only filesystem"
fi

# Check the root filesystem:
if [ ! $READWRITE = yes ]; then
  echo "Checking root filesystem:"
  /sbin/fsck -C -a /
  RETVAL=$?
  # An error code of 2 or higher will require a reboot.
  if [ $RETVAL -ge 2 ]; then
    # An error code equal to or greater than 4 means that some errors
    # could not be corrected.  This requires manual attention, so we
    # offer a chance to try to fix the problem in single-user mode:
    if [ $RETVAL -ge 4 ]; then
      echo
      echo "***********************************************************"
      echo "*** An error occurred during the root filesystem check. ***"
      echo "***                                                     ***"
      echo "*** If you are using the ext2 filesystem, running       ***"
      echo "*** 'e2fsck -v -y <partition>' might help.              ***"
      echo "***********************************************************"
      echo
      echo "Once you exit the single-user shell, the system will reboot."
      echo
      PS1="(Repair filesystem) \#"; export PS1
      sulogin
    else # With an error code of 2 or 3, reboot the machine automatically:
      echo
      echo "***********************************"
      echo "*** The filesystem was changed. ***"
      echo "*** The system will now reboot. ***"
      echo "***********************************"
      echo
    fi
    echo "Unmounting file systems."
    /sbin/umount -a -r -n
    /sbin/mount -n -o remount,ro /
    echo "Rebooting system."
    sleep 2
    reboot -f
  fi
  # Remount the root filesystem in read-write mode
#  echo "Remounting root device with read-write enabled."
#  /sbin/mount -w -v -n -o remount /
#  if [ $? -gt 0 ] ; then
#    echo
#    echo "Attempt to remount root device as read-write failed!  This is going to"
#    echo "cause serious problems."
#    echo 
#    echo "Please press ENTER to continue, then reboot and use one of the above methods to"
#    echo -n "get into your machine and start looking for the problem."
#    read junk;
#  fi
else
  echo "Testing root filesystem status:  read-write filesystem"
    echo
    echo "*** ERROR: Root partition has already been mounted read-write. Cannot check!"
    echo
    echo "This will fix the problem *AND* eliminate this annoying message. :^)"
    echo
    echo -n "Press ENTER to continue. "
    read junk;
fi # Done checking root filesystem

# Set the system time from the hardware clock using hwclock --hctosys.
if [ -x /sbin/hwclock ]; then
  # Check for a broken motherboard RTC clock (where ioports for rtc are
  # unknown) to prevent hwclock causing a hang:
  if ! grep -q -w rtc /proc/ioports ; then
    CLOCK_OPT="--directisa"
  fi
  /sbin/hwclock $CLOCK_OPT --localtime --hctosys
fi

# Check all the non-root filesystems:
echo "Checking non-root filesystems:"
/sbin/fsck $FORCEFSCK -C -R -A -a

# mount non-root file systems in fstab (but not NFS or SMB 
# because TCP/IP is not yet configured, and not proc because
# that has already been mounted):
/sbin/mount -a -v -n -t nonfs,nosmbfs,noproc
mount -tvfat -n -o noatime,sync,dirsync,uid=deka,gid=users,iocharset=koi8-r -v /dev/hdc5 /Data

mkfs -q /dev/hdc2
mount /dev/hdc2 /tmp
chmod 1777 /tmp
mkdir -p /tmp/run/proftpd

# Enable swapping again.  This is needed in case a swapfile is used,
# as it can't be enabled until the filesystem it resides on has been
# mounted read-write.
/sbin/swapon -a

grep -q "/dev/root / .* rw[, ]" /etc/mtab && {
  # Clean up some temporary files:
  ( cd /var/log/setup/tmp && rm -rf * )
  /bin/rm -f /var/run/utmp /var/run/*pid /etc/nologin /var/run/lpd* \
    /var/run/ppp* /etc/dhcpc/*.pid /etc/forcefsck /etc/fastboot

  # Create a fresh utmp file:
  touch /var/run/utmp
  chown root.utmp /var/run/utmp
  chmod 664 /var/run/utmp
}

hdparm -S 2 /dev/hda
